//
//  ViewController.swift
//  BLDCMotor
//
//  Created by Phidgets on 2018-01-25.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = BLDCMotor()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var velocityLabel: UILabel!
    @IBOutlet var positionLabel: UILabel!
    @IBOutlet var targetVelocitySlider: UISlider!
    @IBOutlet var targetVelocityLabel: UILabel!
    @IBOutlet var accelerationSlider: UISlider!
    @IBOutlet var accelerationLabel: UILabel!
    @IBOutlet var brakingStrengthSlider: UISlider!
    @IBOutlet var brakingStrengthLabel: UILabel!
    @IBOutlet var stallVelocitySlider: UISlider!
    @IBOutlet var stallVelocityLabel: UILabel!
    @IBOutlet var rescaleText: UITextField!
    @IBOutlet var stackView: UIStackView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.velocityUpdate.addHandler(velocityupdate_handler)
            let _ = ch.positionChange.addHandler(positionchange_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! BLDCMotor
            
            let minTargetVelocity = try Float(attachedDevice.getMaxVelocity() * -1)
            let maxTargetVelocity = try Float(attachedDevice.getMaxVelocity())
            let minAcceleration = try Float(attachedDevice.getMinAcceleration())
            let maxAcceleration = try Float(attachedDevice.getMaxAcceleration())
            let acceleration = try Float(attachedDevice.getAcceleration())
            let minBrakingStrength = try Float(attachedDevice.getMinBrakingStrength())
            let maxBrakingStrength = try Float(attachedDevice.getMaxBrakingStrength())
            let brakingStrength = try Float(attachedDevice.getTargetBrakingStrength())
            let minStallVelocity = try Float(attachedDevice.getMinStallVelocity())
            let maxStallVelocity = try Float(attachedDevice.getMaxStallVelocity())
            let stallVelocity = try Float(attachedDevice.getStallVelocity())
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.targetVelocitySlider.minimumValue = minTargetVelocity
                self.targetVelocitySlider.maximumValue = maxTargetVelocity
                self.targetVelocitySlider.value = 0
                self.targetVelocityLabel.text = "0.00"
                
                self.accelerationSlider.minimumValue = minAcceleration
                self.accelerationSlider.maximumValue = maxAcceleration
                self.accelerationSlider.value = acceleration
                self.accelerationLabel.text = "\(acceleration)"
                
                self.brakingStrengthSlider.minimumValue = minBrakingStrength
                self.brakingStrengthSlider.maximumValue = maxBrakingStrength
                self.brakingStrengthSlider.value = brakingStrength
                self.brakingStrengthLabel.text = "\(brakingStrength)"
                
                self.stallVelocitySlider.minimumValue = minStallVelocity
                self.stallVelocitySlider.maximumValue = maxStallVelocity
                self.stallVelocitySlider.value = stallVelocity
                self.stallVelocityLabel.text = "\(stallVelocity)"
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func positionchange_handler(sender: BLDCMotor, position: Double){
        DispatchQueue.main.async {
            self.positionLabel.text = "\(position)"
        }
    }
    
    func velocityupdate_handler(sender: BLDCMotor, velocity: Double){
        DispatchQueue.main.async {
            self.velocityLabel.text = String(format:"%.2f", velocity)
        }
    }

    //GUI controls
    @IBAction func setTargetVelocity(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setTargetVelocity(Double(slider.value))
            DispatchQueue.main.async {
                self.targetVelocityLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setAcceleration(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setAcceleration(Double(slider.value))
            DispatchQueue.main.async {
                self.accelerationLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setBrakingStrength(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setTargetBrakingStrength(Double(slider.value))
            DispatchQueue.main.async {
                self.brakingStrengthLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setStallVelocity(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setStallVelocity(Double(slider.value))
            DispatchQueue.main.async {
                self.stallVelocityLabel.text = String(format:"%.2f", Double(slider.value))
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setRescaleFactor(_ sender: Any) {
        do{
            self.view.endEditing(true)
            let rescale = Double(rescaleText.text!)
            if(rescale != nil){
                try ch.setRescaleFactor(rescale!)
                let minStallVelocity = try Float(ch.getMinStallVelocity())
                let maxStallVelocity = try Float(ch.getMaxStallVelocity())
                let stallVelocity = try Float(ch.getStallVelocity())
                let position = try Float(ch.getPosition())
                
                DispatchQueue.main.async {
                    self.stallVelocitySlider.minimumValue = minStallVelocity
                    self.stallVelocitySlider.maximumValue = maxStallVelocity
                    self.stallVelocitySlider.value = stallVelocity
                    self.stallVelocityLabel.text = String(format:"%.2f", stallVelocity)
                    self.positionLabel.text = "\(position)"
                }
                
            }
            else{
                outputErrorMessage(errorTitle: "Error", errorMessage: "Rescale factor not valid")
                let currentRescale = try ch.getRescaleFactor()
                DispatchQueue.main.async {
                    self.rescaleText.text = "\(currentRescale)"
                }
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    

    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    

}

